@@ -46,20 +46,37 @@
 //! # }
 //! ```
 
+#![cfg_attr(all(not(feature = "std"), not(test)), no_std)]
 #![warn(missing_docs)]
-#![warn(rust_2018_idioms)]
 
 // If neither is specified, the crate is essentially empty.
 #[cfg(all(not(feature = "picnic"), not(feature = "picnic3")))]
 compile_error!("One of the features \"picnic\" and \"picnic3\" is required.");
 
-use core::convert::TryFrom;
-use core::fmt::{Debug, Formatter};
+#[cfg(not(feature = "std"))]
+extern crate alloc;
+
+#[cfg(not(feature = "std"))]
+use alloc::{
+    fmt::{self, Debug},
+    format, vec,
+    vec::Vec,
+};
+
+#[cfg(feature = "std")]
+use std::fmt::{self, Debug};
+
 use core::marker::PhantomData;
 use paste::paste;
 use picnic_sys::*;
 pub use signature::{self, Error};
 
+#[cfg(feature = "serialization")]
+use serde::{de, Deserialize, Deserializer, Serialize, Serializer};
+
+#[cfg(feature = "subtle")]
+use subtle::{Choice, ConstantTimeEq};
+
 mod wrapper;
 use crate::wrapper::*;
 
@@ -75,9 +92,7 @@ pub trait Parameters {
     const PUBLIC_KEY_SIZE: usize;
 
     /// Retrieve name of the parameter set
-    fn parameter_name() -> String {
-        wrapper::parameter_name(Self::PARAM).into()
-    }
+    fn parameter_name() -> &'static str;
 }
 
 /// Extension of the [signature::Signer] trait that allows to retrieve to corresponding verifier
@@ -98,6 +113,8 @@ macro_rules! define_params {
     ($name:ident, $param:ident, $realparam:expr) => {
         paste! {
             #[doc = $name " parameters"]
+            #[derive(Clone, Debug, PartialEq, Eq)]
+            #[cfg_attr(feature = "serialization", derive(Serialize, Deserialize))]
             pub struct $name {}
 
             impl Parameters for $name {
@@ -105,6 +122,10 @@ macro_rules! define_params {
                 const MAX_SIGNATURE_SIZE: usize = [<PICNIC_SIGNATURE_SIZE_ $param>];
                 const PRIVATE_KEY_SIZE: usize = [<PICNIC_PRIVATE_KEY_SIZE_ $param>];
                 const PUBLIC_KEY_SIZE: usize = [<PICNIC_PUBLIC_KEY_SIZE_ $param>];
+
+                fn parameter_name() -> &'static str {
+                    "$param"
+                }
             }
 
             #[doc = "Signing key for " $name]
@@ -159,7 +180,10 @@ define_params!(Picnic3L5, Picnic3_L5, picnic_params_t::Picnic3_L5);
 /// While storing signatures in arrays is possible, their size varies even for the same parameter
 /// set.
 #[derive(Debug, Clone, Eq, PartialEq)]
-pub struct DynamicSignature(Vec<u8>);
+#[cfg_attr(feature = "serialization", derive(Serialize, Deserialize))]
+pub struct DynamicSignature(
+    #[cfg_attr(feature = "serialization", serde(with = "serde_bytes"))] Vec<u8>,
+);
 
 impl AsRef<[u8]> for DynamicSignature {
     fn as_ref(&self) -> &[u8] {
@@ -279,8 +303,8 @@ impl<P> Debug for SigningKey<P>
 where
     P: Parameters,
 {
-    fn fmt(&self, fmt: &mut Formatter<'_>) -> std::fmt::Result {
-        fmt.debug_struct(format!("SigningKey<{}>", P::parameter_name()).as_str())
+    fn fmt(&self, fmt: &mut fmt::Formatter<'_>) -> fmt::Result {
+        fmt.debug_struct(&format!("SigningKey<{}>", P::parameter_name()))
             .field("data", &self.data)
             .finish()
     }
@@ -298,6 +322,43 @@ where
 
 impl<P> Eq for SigningKey<P> where P: Parameters {}
 
+#[cfg(feature = "subtle")]
+impl<P> ConstantTimeEq for SigningKey<P>
+where
+    P: Parameters,
+{
+    fn ct_eq(&self, other: &Self) -> Choice {
+        self.data.as_ref().data[..P::PRIVATE_KEY_SIZE]
+            .ct_eq(&other.data.as_ref().data[..P::PRIVATE_KEY_SIZE])
+    }
+}
+
+#[cfg(feature = "serialization")]
+impl<P> Serialize for SigningKey<P>
+where
+    P: Parameters,
+{
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        serialize(self, serializer)
+    }
+}
+
+#[cfg(feature = "serialization")]
+impl<'de, P> Deserialize<'de> for SigningKey<P>
+where
+    P: Parameters,
+{
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+    where
+        D: Deserializer<'de>,
+    {
+        deserialize(deserializer)
+    }
+}
+
 /// Verification key generic over the parameters
 #[derive(Clone)]
 pub struct VerificationKey<P: Parameters> {
@@ -359,8 +420,8 @@ impl<P> Debug for VerificationKey<P>
 where
     P: Parameters,
 {
-    fn fmt(&self, fmt: &mut Formatter<'_>) -> std::fmt::Result {
-        fmt.debug_struct(format!("VerificationKey<{}>", P::parameter_name()).as_str())
+    fn fmt(&self, fmt: &mut fmt::Formatter<'_>) -> fmt::Result {
+        fmt.debug_struct(&format!("VerificationKey<{}>", P::parameter_name()))
             .field("data", &self.data)
             .finish()
     }
@@ -378,6 +439,32 @@ where
 
 impl<P> Eq for VerificationKey<P> where P: Parameters {}
 
+#[cfg(feature = "serialization")]
+impl<P> Serialize for VerificationKey<P>
+where
+    P: Parameters,
+{
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        serialize(self, serializer)
+    }
+}
+
+#[cfg(feature = "serialization")]
+impl<'de, P> Deserialize<'de> for VerificationKey<P>
+where
+    P: Parameters,
+{
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+    where
+        D: Deserializer<'de>,
+    {
+        deserialize(deserializer)
+    }
+}
+
 /// Signing key
 #[derive(Clone, Debug)]
 pub struct DynamicSigningKey {
@@ -450,6 +537,26 @@ impl PartialEq for DynamicSigningKey {
 
 impl Eq for DynamicSigningKey {}
 
+#[cfg(feature = "serialization")]
+impl Serialize for DynamicSigningKey {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        serialize(self, serializer)
+    }
+}
+
+#[cfg(feature = "serialization")]
+impl<'de> Deserialize<'de> for DynamicSigningKey {
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+    where
+        D: Deserializer<'de>,
+    {
+        deserialize(deserializer)
+    }
+}
+
 /// Verification key
 #[derive(Clone, Debug)]
 pub struct DynamicVerificationKey {
@@ -502,12 +609,81 @@ impl PartialEq for DynamicVerificationKey {
 
 impl Eq for DynamicVerificationKey {}
 
+#[cfg(feature = "serialization")]
+impl Serialize for DynamicVerificationKey {
+    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
+    where
+        S: Serializer,
+    {
+        serialize(self, serializer)
+    }
+}
+
+#[cfg(feature = "serialization")]
+impl<'de> Deserialize<'de> for DynamicVerificationKey {
+    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
+    where
+        D: Deserializer<'de>,
+    {
+        deserialize(deserializer)
+    }
+}
+
+#[cfg(feature = "serialization")]
+fn serialize<T, S>(value: &T, serializer: S) -> Result<S::Ok, S::Error>
+where
+    T: AsRef<[u8]>,
+    S: Serializer,
+{
+    serde_bytes::serialize(value.as_ref(), serializer)
+}
+
+#[cfg(feature = "serialization")]
+fn deserialize<'de, T, D>(deserializer: D) -> Result<T, D::Error>
+where
+    T: for<'a> TryFrom<&'a [u8]>,
+    D: Deserializer<'de>,
+{
+    struct BytesVisitor<S>(PhantomData<S>);
+
+    impl<'de, S> de::Visitor<'de> for BytesVisitor<S>
+    where
+        S: for<'a> TryFrom<&'a [u8]>,
+    {
+        type Value = S;
+
+        fn expecting(&self, formatter: &mut fmt::Formatter) -> fmt::Result {
+            write!(formatter, "a byte array")
+        }
+
+        fn visit_bytes<E>(self, v: &[u8]) -> Result<Self::Value, E>
+        where
+            E: de::Error,
+        {
+            match S::try_from(v) {
+                Ok(t) => Ok(t),
+                Err(_) => Err(de::Error::invalid_value(de::Unexpected::Bytes(v), &self)),
+            }
+        }
+
+        fn visit_borrowed_bytes<E>(self, v: &'de [u8]) -> Result<Self::Value, E>
+        where
+            E: de::Error,
+        {
+            match S::try_from(v) {
+                Ok(t) => Ok(t),
+                Err(_) => Err(de::Error::invalid_value(de::Unexpected::Bytes(v), &self)),
+            }
+        }
+    }
+
+    deserializer.deserialize_bytes(BytesVisitor::<T>(PhantomData))
+}
+
 #[cfg(test)]
 mod test {
     #[cfg(feature = "picnic")]
     use crate::{PicnicL1FSSigningKey, PicnicL1FullSigningKey, PicnicL1FullVerificationKey};
-    #[cfg(feature = "picnic")]
-    use std::convert::TryFrom;
 
     #[cfg(feature = "picnic")]
     #[test]
